/* Author: Jeff Dalton <J.Dalton@ed.ac.uk> &
 *         Stephen Potter <stephenp@aiai.ed.ac.uk>
 * Last Updated: Mon Sep 02 17:15 2002 by Stephen Potter
 * Copyright: (c) 2001-2, AIAI, University of Edinburgh
 */

package ix.jabber;

import java.io.*;

import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;
import org.jdom.output.XMLOutputter;

import org.w3c.dom.*;
//import com.sun.xml.tree.*;
//import sunlabs.brazil.util.http.*;
import java.util.*;
import java.net.*;

import ix.util.xml.XML;

import ix.util.*;

import org.jabber.jabberbeans.*;
import org.jabber.jabberbeans.Extension.*;
import org.jabber.jabberbeans.serverside.*;
import org.jabber.jabberbeans.util.*;


/*
 * A class that encapsulates the knowledge needed for
 * sending and receiving messages using the Jabber protocol.
 */

public class JabberCommunicationStrategy 
       implements IPC.CommunicationStrategy {

    public ConnectionBean connectionBean;
    public String jabberServer;
    public String username;
    public String resource;
    public String password;
    public String presence;
    public IPC.MessageListener ipcListener = null;

    public JabberCommunicationStrategy() {
    }

    /**
     * Sends an Issue or Report to a named jabber agent by converting
     * it to XML and calling the <code>sendText</code> method.
     */
    public void sendObject(Object destination, Object contents) throws IPC.IPCException {
	String toName = (String)destination;
	if(toName.lastIndexOf("@")==-1){ // ie if no '@' in string...
	    throw new IPC.IPCException("Recipient address is not of the form name@jabber-server\nMessage not sent");
	}
	else{
	    sendText(toName, XML.objectToXMLString(contents));
	}
    }

    /**
     * Sends a string to a named I-X panel.
     */
    public void sendText(String toName, String contents){

	MessageBuilder mb=new MessageBuilder();
	Message msg;
	//set destination user.

	// assume that toName is of the form username@jabberserver...
	
	int atInd = toName.lastIndexOf("@");
		 
	String name = toName.substring(0,atInd);
	String server = toName.substring(atInd+1);

	Debug.noteln("\nSending to:"+name+" at: "+server);

	// note session information here...assumes that
	// all relevant I-X panels in this session will be
	// using the same resource name.
	mb.setToAddress(new JID(name, server, resource));
	
	mb.setBody(contents);
	try
	    {
		msg=(Message)mb.build();
	    }
	catch (InstantiationException e)
	    {
		//build failed ?
		System.out.println("error creating message packet:");
		System.out.println(e.toString());
		return;
	    }

	//send message
	connectionBean.send(msg);
    }


    // logon to the named jabber server...
    public void logon(){
	
	InfoQueryBuilder iqb=new InfoQueryBuilder();
	InfoQuery iq;
	//and the auth data builder
	IQAuthExtensionBuilder iqAuthb=new IQAuthExtensionBuilder();

	//we are setting info
	iqb.setType("set");

	//user,password & resource are from the parameter list (or defaults).
	iqAuthb.setUsername(username);
	iqAuthb.setPassword(password);
	iqAuthb.setResource(resource);

	//build the Auth data
	try
	{
	    iqb.addExtension(iqAuthb.build());
	}
	catch (InstantiationException e)
	{
	    //building failed ?
	    java.lang.System.out.println("Fatal Error on Auth object build:");
	    java.lang.System.out.println(e.toString());
	    return;
	}

	//build the iq packet
	try
	{
	    //build the full InfoQuery packet
	    iq=(InfoQuery)iqb.build();
	}
	catch (InstantiationException e)
	{
	    //building failed ?
	    java.lang.System.out.println("Fatal Error on IQ object build:");
	    java.lang.System.out.println(e.toString());
	    return ;
	}

	//send the InfoQuery packet to the server to log in
	connectionBean.send(iq);
	
	java.lang.System.out.println("Send requested for IQ");
	
    }


    /**
     * Initiate a jabber receiver object and set up for receiving messages.
     */
    public void setupServer(Object destination, 
			    IPC.MessageListener listener) {

	InetAddress host;
	connectionBean = new ConnectionBean();
	ipcListener = listener;

	if(Parameters.haveParameter("jabber-server")){
	    jabberServer = Parameters.getParameter("jabber-server");
	}
	else
	    jabberServer = "akt.aiai.ed.ac.uk"; // default

	if(Parameters.haveParameter("jabber-username")){
	    username = Parameters.getParameter("jabber-username");
	}
	else
	    username = "test"; // default

	if(Parameters.haveParameter("jabber-password")){
	    password = Parameters.getParameter("jabber-password");
	}
	else
	    password = "jabber"; // default

	if(Parameters.haveParameter("jabber-resource")){
	    resource = Parameters.getParameter("jabber-resource");
	    // don't allow a null resource name:
	    if(resource.equalsIgnoreCase("")) resource = "I-X";
	}
	else
	    resource = "I-X"; //default


	try{
	    connectionBean.addPacketListener(new MyPacketListener());
	    connectionBean.addConnectionListener(new MyConnectionListener());

	    //	    host = InetAddress.getByName("akt.aiai.ed.ac.uk");
	    host = InetAddress.getByName(jabberServer);
	    System.out.print("got host\n");
	    connectionBean.connect(host);
	    System.out.print("connected!\n");	    
	    logon();
	}
	catch (Exception e){
	    System.out.print(e);
	}

	PresenceBuilder pb=new PresenceBuilder();
	    
	//this is presence to the irc transport on jabber.org. This will log
	//you into the #jabber channel. The best way to see this working is to
	//be on the jabber channel yourself (it is on irc.openprojects.net)
	//	JID dest=new JID("#jabberbeans","irc.jabber.org",
	//		"jtest");
	//pb.setToAddress(dest);

	if(Parameters.haveParameter("jabber-presence")){
	    presence = Parameters.getParameter("jabber-presence");
	    
	    // in case of empty/null parameter declaration:
	    if(presence.equalsIgnoreCase("")) presence = "Online";
	}
	else
	    presence = "Online"; // default

	String show = "";

	if(presence.equalsIgnoreCase("Away") || 
	   presence.equalsIgnoreCase("Lunch") ||
	   presence.equalsIgnoreCase("Bank"))
	    show = "away";
	if(presence.equalsIgnoreCase("Busy") || 
	   presence.equalsIgnoreCase("Working") ||
	   presence.equalsIgnoreCase("Mad"))
	    show = "dnd";
	if(presence.equalsIgnoreCase("Extended Away") || 
	   presence.equalsIgnoreCase("Gone Home") ||
	   presence.equalsIgnoreCase("Gone to Work") ||
	   presence.equalsIgnoreCase("Sleeping"))
	    show = "xa";
	if(presence.equalsIgnoreCase("Wants to Chat"))
	    show = "chat";

	pb.setStatus(presence);
	pb.setStateShow(show);

	try
	    {
		//build and send presence
		connectionBean.send(pb.build());
	    }
	catch (InstantiationException e)
	    {
		//building failed ?
		System.out.println
		    ("Fatal Error on Presence object build:");
		System.out.println(e.toString());
		return ;
	    }
    
    }


    static class MyConnectionListener implements ConnectionListener
    {
	public void connected(ConnectionEvent ce)
	{
	    System.out.println("CLDebug: Connected!");
	}
	public void disconnected(ConnectionEvent ce)
	{
	    System.out.println("CLDebug: Disconnected");
	}
	public void connecting(ConnectionEvent ce)
	{
	    System.out.println("CLDebug: Connecting...");
	    }
	public void connectFailed(ConnectionEvent ce)
	{
	    System.out.println("CLDebug: Connect Failed");
	}
	public void connectionChanged(ConnectionEvent ce)
	{
	    System.out.println("CLDebug: Connection Changed");
	}
    }


    class MyPacketListener implements PacketListener {

	public void receivedPacket(PacketEvent pe)
	{
	    Packet p = pe.getPacket();

	    System.out.println("PLDebug: RECEIVED PACKET!!! : " 
			       + p.toString());

	    if(p instanceof Message){

		try{
		    Message m = (Message)p;
	
		    // normal, chat, or headline
		    System.out.println("Message of type: " + m.getType());
		    System.out.println("\treceived from: " + 
				       m.getFromAddress().toString());
		    System.out.println("\tbody: " + m.getBody());
	    
		    Object contents = XML.objectFromXML(m.getBody());

		    // and notify the ipcListener:
		    ipcListener.messageReceived(new IPC.BasicInputMessage(contents));
		}
		catch (Exception e){

		    Debug.noteException(e);

		}

	    }

	    if(p instanceof Presence){
		try{
		    Presence pr = (Presence)p;
		    System.out.println("\nPresence: " 
				       + pr.getFromAddress().toString() 
				       + " --- " + pr.getStatus() + "\n");
		}
		catch (Exception e){
		    Debug.noteException(e);
		}
	    }		
	}

	public void sentPacket(PacketEvent pe)
	{
	    System.out.println("PLDebug: Sent Packet : " + 
			       pe.getPacket().toString());
	}
	public void sendFailed(PacketEvent pe)
	{
	    System.out.println("PLDebug: Send Failed Packet : " + 
			       pe.getPacket().toString());
	}
    }

}
